@@ -6,7 +6,6 @@ import os
 import shutil
 import logging
 import click
-import click_log
 from policy_sentry.querying.all import get_all_service_prefixes
 from policy_sentry.shared.awsdocs import (
     update_html_docs_directory,
@@ -22,9 +21,9 @@ from policy_sentry.shared.constants import (
     BUNDLED_DATASTORE_FILE_PATH,
     BUNDLED_DATA_DIRECTORY,
 )
+from policy_sentry import set_stream_logger
 
 logger = logging.getLogger(__name__)
-click_log.basic_config(logger)
 
 
 @click.command(name="initialize", short_help="Create a local datastore to store AWS IAM information.")
@@ -51,11 +50,18 @@ click_log.basic_config(logger)
     help="Build the IAM data file from the HTML files rather than copying the data file from "
     "the python package. Defaults to false",
 )
-@click_log.simple_verbosity_option(logger)
-def initialize_command(access_level_overrides_file, fetch, build):
+@click.option(
+    '--verbose', '-v',
+    type=click.Choice(['critical', 'error', 'warning', 'info', 'debug'],
+    case_sensitive=False))
+def initialize_command(access_level_overrides_file, fetch, build, verbose):
     """
     CLI command for initializing the local data file
     """
+    if verbose:
+        log_level = getattr(logging, verbose.upper())
+        set_stream_logger(level=log_level)
+
     initialize(access_level_overrides_file, fetch, build)
 
 
@@ -93,13 +99,11 @@ def initialize(access_level_overrides_file=None, fetch=False, build=False):
         shutil.copy(BUNDLED_DATASTORE_FILE_PATH, database_path)
 
     # --fetch: wget the AWS IAM Actions, Resources and Condition Keys pages and store them locally.
-    # if --build and --fetch are both supplied, just do --fetch
     if fetch:
         # `wget` the html docs to the local directory
         update_html_docs_directory(LOCAL_HTML_DIRECTORY_PATH)
-        create_database(CONFIG_DIRECTORY, overrides_file)
 
-    # initialize --build
+    # --build
     if build or access_level_overrides_file or fetch:
         create_database(CONFIG_DIRECTORY, overrides_file)
         print("Created the database!")
@@ -135,7 +139,6 @@ def create_policy_sentry_config_directory():
 def create_html_docs_directory():
     """
     Copies the HTML files from the pip package over to its own folder in the CONFIG_DIRECTORY.
-    Also copies over the links.yml file, which is a mapping of services and relevant HTML links in the AWS docs.
     Essentially:
     mkdir -p ~/.policy_sentry/data/docs
     cp -r $MODULE_DIR/policy_sentry/shared/data/docs ~/.policy_sentry/data/docs
