@@ -1,10 +1,10 @@
 """
 Allow users to use specific pre-compiled queries against the action, arn, and condition tables from command line.
 """
+import os
 import json
 import logging
 import click
-import click_log
 import yaml
 from policy_sentry.util.access_levels import transform_access_level_text
 from policy_sentry.querying.all import get_all_service_prefixes
@@ -17,17 +17,19 @@ from policy_sentry.querying.actions import (
     get_actions_for_service,
     get_actions_with_access_level,
     get_action_data,
-    get_actions_that_support_wildcard_arns_only,
     get_actions_matching_condition_key,
-    get_actions_at_access_level_that_support_wildcard_arns_only,
+    get_actions_with_arn_type_and_access_level,
+    get_actions_matching_arn_type
 )
 from policy_sentry.querying.conditions import (
     get_condition_keys_for_service,
     get_condition_key_details,
 )
+from policy_sentry.shared.constants import DATASTORE_FILE_PATH, LOCAL_DATASTORE_FILE_PATH
+from policy_sentry import set_stream_logger
 
 logger = logging.getLogger(__name__)
-click_log.basic_config(logger)
+iam_definition_path = DATASTORE_FILE_PATH
 
 
 @click.group()
@@ -62,11 +64,10 @@ def query():
     help="Supply a condition key to show a list of all IAM actions that support the condition key.",
 )
 @click.option(
-    "--wildcard-only",
-    is_flag=True,
+    "--resource-type",
+    type=str,
     required=False,
-    help="Show the IAM actions that only support "
-    "wildcard resources - i.e., cannot support ARNs in the resource block.",
+    help="Supply a resource type to show a list of all IAM actions that support the resource type.",
 )
 @click.option(
     "--fmt",
@@ -75,16 +76,29 @@ def query():
     required=False,
     help='Format output as YAML or JSON. Defaults to "yaml"',
 )
-@click_log.simple_verbosity_option(logger)
-def action_table(name, service, access_level, condition, wildcard_only, fmt):
+@click.option(
+    '--verbose', '-v',
+    type=click.Choice(['critical', 'error', 'warning', 'info', 'debug'],
+    case_sensitive=False))
+def action_table(name, service, access_level, condition, resource_type, fmt, verbose):
     """Query the Action Table from the Policy Sentry database"""
-    query_action_table(name, service, access_level, condition, wildcard_only, fmt)
+    if verbose:
+        log_level = getattr(logging, verbose.upper())
+        set_stream_logger(level=log_level)
+
+    query_action_table(name, service, access_level, condition, resource_type, fmt)
 
 
 def query_action_table(
-    name, service, access_level, condition, wildcard_only, fmt="json"
+    name, service, access_level, condition, resource_type, fmt="json"
 ):
-    """Query the Action Table from the Policy Sentry database. Use this one when leveraging Policy Sentry as a library."""
+    """Query the Action Table from the Policy Sentry database.
+    Use this one when leveraging Policy Sentry as a library."""
+    if os.path.exists(LOCAL_DATASTORE_FILE_PATH):
+        logger.info(f"Using the Local IAM definition: {LOCAL_DATASTORE_FILE_PATH}. To leverage the bundled definition instead, remove the folder $HOME/.policy_sentry/")
+    else:
+        # Otherwise, leverage the datastore inside the python package
+        logger.debug("Leveraging the bundled IAM Definition.")
     # Actions on all services
     if service == "all":
         all_services = get_all_service_prefixes()
@@ -105,7 +119,7 @@ def query_action_table(
             print(yaml.dump(output)) if fmt == "yaml" else [
                 print(item) for item in output
             ]
-    elif name is None and access_level and not wildcard_only:
+    elif name is None and access_level and not resource_type:
         print(
             f"All IAM actions under the {service} service that have the access level {access_level}:"
         )
@@ -114,14 +128,12 @@ def query_action_table(
         print(yaml.dump(output)) if fmt == "yaml" else [
             print(json.dumps(output, indent=4))
         ]
-    elif name is None and access_level and wildcard_only:
+    elif name is None and access_level and resource_type:
         print(
-            f"{service} {access_level.upper()} actions that must use wildcards in the resources block:"
+            f"{service} {access_level.upper()} actions that have the resource type {resource_type.upper()}:"
         )
         access_level = transform_access_level_text(access_level)
-        output = get_actions_at_access_level_that_support_wildcard_arns_only(
-            service, access_level
-        )
+        output = get_actions_with_arn_type_and_access_level(service, resource_type, access_level)
         print(yaml.dump(output)) if fmt == "yaml" else [
             print(json.dumps(output, indent=4))
         ]
@@ -136,11 +148,11 @@ def query_action_table(
         ]
     # Get a list of IAM Actions under the service that only support resources = "*"
     # (i.e., you cannot restrict it according to ARN)
-    elif wildcard_only:
+    elif resource_type:
         print(
-            f"IAM actions under {service} service that support wildcard resource values only:"
+            f"IAM actions under {service} service that have the resource type {resource_type}:"
         )
-        output = get_actions_that_support_wildcard_arns_only(service)
+        output = get_actions_matching_arn_type(service, resource_type)
         print(yaml.dump(output)) if fmt == "yaml" else [
             print(json.dumps(output, indent=4))
         ]
@@ -183,14 +195,25 @@ def query_action_table(
     required=False,
     help='Format output as YAML or JSON. Defaults to "yaml"',
 )
-@click_log.simple_verbosity_option(logger)
-def arn_table(name, service, list_arn_types, fmt="json"):
+@click.option(
+    '--verbose', '-v',
+    type=click.Choice(['critical', 'error', 'warning', 'info', 'debug'],
+    case_sensitive=False))
+def arn_table(name, service, list_arn_types, fmt="json", verbose=None):
     """Query the ARN Table from the Policy Sentry database"""
+    if verbose:
+        log_level = getattr(logging, verbose.upper())
+        set_stream_logger(level=log_level)
     query_arn_table(name, service, list_arn_types, fmt)
 
 
 def query_arn_table(name, service, list_arn_types, fmt):
     """Query the ARN Table from the Policy Sentry database. Use this one when leveraging Policy Sentry as a library."""
+    if os.path.exists(LOCAL_DATASTORE_FILE_PATH):
+        logger.info(f"Using the Local IAM definition: {LOCAL_DATASTORE_FILE_PATH}. To leverage the bundled definition instead, remove the folder $HOME/.policy_sentry/")
+    else:
+        # Otherwise, leverage the datastore inside the python package
+        logger.debug("Leveraging the bundled IAM Definition.")
     # Get a list of all RAW ARN formats available through the service.
     if name is None and list_arn_types is False:
         output = get_raw_arns_for_service(service)
@@ -229,14 +252,26 @@ def query_arn_table(name, service, list_arn_types, fmt):
     required=False,
     help='Format output as YAML or JSON. Defaults to "yaml"',
 )
-@click_log.simple_verbosity_option(logger)
-def condition_table(name, service, fmt):
+@click.option(
+    '--verbose', '-v',
+    type=click.Choice(['critical', 'error', 'warning', 'info', 'debug'],
+    case_sensitive=False))
+def condition_table(name, service, fmt, verbose):
     """Query the condition table from the Policy Sentry database"""
+    if verbose:
+        log_level = getattr(logging, verbose.upper())
+        set_stream_logger(level=log_level)
     query_condition_table(name, service, fmt)
 
 
 def query_condition_table(name, service, fmt="json"):
-    """Query the condition table from the Policy Sentry database. Use this one when leveraging Policy Sentry as a library."""
+    """Query the condition table from the Policy Sentry database.
+    Use this one when leveraging Policy Sentry as a library."""
+    if os.path.exists(LOCAL_DATASTORE_FILE_PATH):
+        logger.info(f"Using the Local IAM definition: {LOCAL_DATASTORE_FILE_PATH}. To leverage the bundled definition instead, remove the folder $HOME/.policy_sentry/")
+    else:
+        # Otherwise, leverage the datastore inside the python package
+        logger.debug("Leveraging the bundled IAM Definition.")
     # Get a list of all condition keys available to the service
     if name is None:
         output = get_condition_keys_for_service(service)
