@@ -9,7 +9,7 @@ To have granular control of specific IAM policies, we must use wildcards on IAM
 
 This is typically performed by humans by reducing policies to s3:Get*, ec2:Describe*, and other approaches of the sort.
 
-Netflix's PolicyUniverse has address
+Netflix's PolicyUniverse has addressed this in their minimization code. We borrowed this logic from their code and modified it a bit to fit our needs.
 
 https://aws.amazon.com/iam/faqs/
 Q: How many policies can I attach to an IAM role?
@@ -22,13 +22,16 @@ Q: How many policies can I attach to an IAM role?
 * The size of each managed policy cannot exceed 6,144 characters.
 """
 import logging
+import functools
 
 # from policyuniverse.expander_minimizer import _get_prefixes_for_action
 
 logger = logging.getLogger(__name__)
 
+
 # Borrowed from policyuniverse to reduce size
 # https://github.com/Netflix-Skunkworks/policyuniverse/blob/master/policyuniverse/expander_minimizer.py#L45
+@functools.lru_cache(maxsize=1024)
 def _get_prefixes_for_action(action):
     """
     :param action: iam:cat
